@@ -35,12 +35,12 @@ static const struct ipset_attrname createattr2name[] = {
 	[IPSET_ATTR_PROTO]	= { .name = "PROTO" },
 	[IPSET_ATTR_CADT_FLAGS]	= { .name = "CADT_FLAGS" },
 	[IPSET_ATTR_CADT_LINENO] = { .name = "CADT_LINENO" },
-	[IPSET_ATTR_GC]		= { .name = "GC" },
+	[IPSET_ATTR_INITVAL]	= { .name = "INITVAL" },
 	[IPSET_ATTR_HASHSIZE]	= { .name = "HASHSIZE" },
 	[IPSET_ATTR_MAXELEM]	= { .name = "MAXELEM" },
 	[IPSET_ATTR_MARKMASK]	= { .name = "MARKMASK" },
 	[IPSET_ATTR_NETMASK]	= { .name = "NETMASK" },
-	[IPSET_ATTR_PROBES]	= { .name = "PROBES" },
+	[IPSET_ATTR_BUCKETSIZE]	= { .name = "BUCKETSIZE" },
 	[IPSET_ATTR_RESIZE]	= { .name = "RESIZE" },
 	[IPSET_ATTR_SIZE]	= { .name = "SIZE" },
 	[IPSET_ATTR_ELEMENTS]	= { .name = "ELEMENTS" },
